import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import minimize

#déclaration des variables/paramètres
myFile = 'cinetiqueS2O8.txt'
graphShow = True # False/True
#
V0 = 5.0 #(mL)
C0_S2O3 = 1e-3 #(mol/L)

# initialisaton des variables  
temps = []
Vequiv = []

#lecture du fichier de données
data = open(myFile,'r')
for row in data:
    row = row.split(' ')
    temps.append(float(row[0]))
    Vequiv.append(float(row[1]))

temps  = np.array(temps)
Vequiv = np.array(Vequiv)

t   = temps[0:len(Vequiv)-1]
Veq = Vequiv[0:len(Vequiv)-1]


# calcul des différentes quantiés 
C_I2   = C0_S2O3/(2*V0)*Veq
C_S2O8 = C0_S2O3/(2*V0)*(Vequiv[len(Vequiv)-1]-Veq)
Vitesse = np.diff(C_S2O8)/np.diff(t)

#calcul des paramètres de la régression lineaire
def cost_function(coef): return np.sum((np.log(C_S2O8) - (coef[0]*t+coef[1]))**2)
res = minimize(cost_function,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(res.message)
opt_coef = res.x
residu = np.log(C_S2O8) - (opt_coef[0]*t+opt_coef[1])
r2 = np.corrcoef([np.log(C_S2O8), opt_coef[0]*t+opt_coef[1]])
txt_eqn = f'ln([S208]) : ({opt_coef[0]:1.4f})*t + ({opt_coef[1]:1.4f}), (R^2 = {r2[0,1]**2:1.4f})'
print(txt_eqn)

#calcul des paramètres de la régression lineaire loi v=kA
def cost_functionV(coef): return np.sum((Vitesse - (coef[0]*C_S2O8[0:len(C_S2O8)-1]+coef[1]))**2)
resV = minimize(cost_functionV,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(resV.message)
opt_coefV = resV.x
residuV = Vitesse - (opt_coefV[0]*C_S2O8[0:len(C_S2O8)-1]+opt_coefV[1])
r2V = np.corrcoef([Vitesse, opt_coefV[0]*C_S2O8[0:len(C_S2O8)-1]+opt_coefV[1]])
MV=np.max(np.abs(residuV))
muV=np.mean(residuV)
sigmaV=np.std(residuV)
txt_eqnV = f'V : ({opt_coefV[0]:1.4g})*C + ({opt_coefV[1]:1.4g}), (R^2 = {r2V[0,1]**2:1.4f})'
print(txt_eqnV)

#calcul des paramètres de la régression lineaire méthode différentielle
def cost_functiondif(coef): return np.sum((np.log(-Vitesse) - (coef[0]*np.log(C_S2O8[0:len(C_S2O8)-1])+coef[1]))**2)
resdif = minimize(cost_functiondif,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(resdif.message)
opt_coefdif = resdif.x
residudif = np.log(-Vitesse) - (opt_coefdif[0]*np.log(C_S2O8[0:len(C_S2O8)-1])+opt_coefdif[1])
r2dif = np.corrcoef([np.log(-Vitesse), opt_coefdif[0]*np.log(C_S2O8[0:len(C_S2O8)-1])+opt_coefdif[1]])
Mdif=np.max(np.abs(residudif))
mudif=np.mean(residudif)
sigmadif=np.std(residudif)
txt_eqndif = f'ln(V) : ({opt_coefdif[0]:1.4g})*ln(C) + ({opt_coefdif[1]:1.4g}), (R^2 = {r2dif[0,1]**2:1.4f})'
print(txt_eqndif)


#Graphes et figures
#figure1
fig1 = plt.figure(num=1, figsize=[10,7])
plt.xlabel('temps (min)', fontsize = 12)
plt.ylabel(r'Vequiv (mL)', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$V_{equiv} = f(t)$', fontsize = 16)
plt.axis([0,1200,0,20])
plt.plot(temps, Vequiv, '+', label=r'$V_{equiv}$ (mL)')
#plt.legend(loc='best')
#
##figure2
fig2 = plt.figure(num=2, figsize=[10,7])
plt.xlabel('temps (min)', fontsize = 12)
plt.ylabel(r'[I$_2$] (mol/L)', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$[I_2] = f(t)$', fontsize = 16)
plt.axis([0,60,0,1.1*np.max(C_I2)])
plt.plot(t, C_I2, '+--', label=r'$[I_{2}]$ (mol/L)')
#
##figure3
fig3 = plt.figure(num=3, figsize=[10,7])
plt.xlabel('temps (min)', fontsize = 12)
plt.ylabel(r'$[S_{2}O_{8}]$ (mol/L)', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$[S_{2}O_{8}] = f(t)$', fontsize = 16)
plt.axis([0,60,0,1.1*np.max(C_S2O8)])
plt.plot(t, C_S2O8, '+--', label=r'$[S_{2}O_{8}]$ (mol/L)')
#
##figure4
fig4 = plt.figure(num=4, figsize=[10,7])
ax1 = plt.subplot(211)
ax1.set_xlabel(r'temps (min)', fontsize = 12)
ax1.set_ylabel(r'$ln([S_{2}O_{8}])$', fontsize = 12)
ax1.grid(linestyle='--')
ax1.set_title(r'$ln([S_{2}O_{8}]) = f(t)$', fontsize = 16)
ax1.plot(t, np.log(C_S2O8), '+--', label=r'ln( $[S_{2}O_{8}] )$')
ax1.plot(t,opt_coef[0]*t+opt_coef[1], 'r-' )
ax1.text(t[1],1.02*opt_coef[1],txt_eqn, fontsize=12, color='red')
#
ax2 = plt.subplot(223)
ax2.plot(t,residu, '+' )
ax2.grid(linestyle='--')
ax2.set_xlabel('temps (min)', fontsize = 12)
ax2.set_ylabel('résidus', fontsize = 12)
#
ax3 = plt.subplot(224)
ax3.grid(linestyle='--')
ax3.hist(residu, facecolor='b', alpha=0.75)
ax3.text(0,2.7,f'moyenne = {np.mean(residu):1.4g}', fontsize=10)
ax3.text(0,2.4,f'écart-type = {np.std(residu):1.4f}', fontsize=10)

##figure5
fig5 = plt.figure(num=5, figsize=[10,7])
ax1 = plt.subplot(211)
ax1.set_xlabel(r'ln(C)', fontsize = 12)
ax1.set_ylabel(r'$ln(vitesse)$', fontsize = 12)
ax1.grid(linestyle='--')
ax1.set_title(r'$ln(v) = f(ln(C))$', fontsize = 16)
ax1.plot(np.log(C_S2O8[0:len(C_S2O8)-1]), np.log(abs(Vitesse)), '+--', label=r'vitesse')
ax1.plot(np.log(C_S2O8[0:len(C_S2O8)-1]),opt_coefdif[0]*np.log(C_S2O8[0:len(C_S2O8)-1])+opt_coefdif[1], 'r-' )
ax1.text (-8,-10, txt_eqndif, fontsize=12, color='red')
# ax1.text(0.3*np.max(np.log(C)),.5*(np.min(np.log(abs(Vitesse)))+np.max(np.log(abs(Vitesse))))/2,txt_eqndif, fontsize=12, color='red')
#
ax2 = plt.subplot(223)
ax2.plot(np.log(C_S2O8[0:len(C_S2O8)-1]),residudif, '+' )
ax2.grid(linestyle='--')
ax2.set_xlabel('ln(C)', fontsize = 12)
ax2.set_ylabel('résidus ln(vitesse)', fontsize = 12)
#ax2.axis([0,1.1*np.max(np.log(A)),-1.1*Mdif,1.1*Mdif])
#
ax3 = plt.subplot(224)
ax3.grid(linestyle='--')
ax3.hist(residudif,int(np.rint(len(np.log(C_S2O8))/4)), density=True, facecolor='b', alpha=0.75)
Xdif=np.linspace(-1.1*Mdif,1.1*Mdif,100)
gaussdif = 1/(sigmadif*(2*np.pi)**0.5)*np.exp(-(Xdif-mudif)**2/(2*sigmadif**2))
ax3.plot(Xdif,gaussdif)
ax3.text(-Mdif,1.3*np.max(gaussdif),f'moyenne = {mudif:1.4g}', fontsize=10)
ax3.text(-Mdif,1.2*np.max(gaussdif),f'écart-type = {sigmadif:1.4g}', fontsize=10)
ax3.axis([-1.15*Mdif,1.15*Mdif,0,1.4*np.max(gaussdif)])
#
##figure7
fig5 = plt.figure(num=7, figsize=[10,7])
plt.xlabel('temps (min)', fontsize = 12)
plt.ylabel(r'$1/[S_{2}O_{8}]$ (L/mol)', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$1/[S_{2}O_{8}] = f(t)$', fontsize = 16)
plt.plot(t, 1/C_S2O8, '+--', label=r'$1/[S_{2}O_{8}]$')
#
##figure8
fig6 = plt.figure(num=8, figsize=[10,7])
plt.xlabel('[S2O8]] (mol/L)', fontsize = 12)
plt.ylabel(r'$v$ (L/mol/min)', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$v = f([S2O8]])$', fontsize = 16)
plt.plot(C_S2O8[0:len(C_S2O8)-1], Vitesse, '+--')

##Affichage des figures
if graphShow: plt.show()