import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import minimize

#déclaration des variables/paramètres
myFile =  'solvolyse.txt'
graphShow = True # False/True
#

# initialisaton des variables  
temps = []
conductivité = []

#lecture du fichier de données
data = open(myFile,'r')
for row in data:
    row = row.split('\t')
    temps.append(float(row[0]))
    conductivité.append(float(row[1]))

t = np.array(temps)
conductivité = np.array(conductivité)

# calcul des concentrations et de la vitesse 
Concentration = (conductivité[len(conductivité)-1] - conductivité)/conductivité[len(conductivité)-1]
C=Concentration[0:len(Concentration)-2]
tprime=t[0:len(Concentration)-2]
Vitesse = np.diff(conductivité)/np.diff(t)
vitesseprime = Vitesse[0:len(Concentration)-2]

#calcul des paramètres de la régression lineaire méthode intégrale
def cost_function(coef): return np.sum((np.log(C) - (coef[0]*tprime+coef[1]))**2)
res = minimize(cost_function,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(res.message)
opt_coef = res.x
residu = np.log(C) - (opt_coef[0]*tprime+opt_coef[1])
r2 = np.corrcoef([np.log(C), opt_coef[0]*tprime+opt_coef[1]])
M=np.max(np.abs(residu))
mu=np.mean(residu)
sigma=np.std(residu)
txt_eqn = f'ln(C) : ({opt_coef[0]:1.4f})*t + ({opt_coef[1]:1.4f}), (R^2 = {r2[0,1]**2:1.4f})'
print(txt_eqn)

#calcul des paramètres de la régression lineaire méthode v=kA
def cost_functionV(coef): return np.sum((vitesseprime - (coef[0]*C+coef[1]))**2)
resV = minimize(cost_functionV,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(resV.message)
opt_coefV = resV.x
residuV = vitesseprime - (opt_coefV[0]*C+opt_coefV[1])
r2V = np.corrcoef([vitesseprime, opt_coefV[0]*C+opt_coefV[1]])
MV=np.max(np.abs(residuV))
muV=np.mean(residuV)
sigmaV=np.std(residuV)
txt_eqnV = f'V : ({opt_coefV[0]:1.4g})*C + ({opt_coefV[1]:1.4g}), (R^2 = {r2V[0,1]**2:1.4f})'
print(txt_eqnV)

#calcul des paramètres de la régression lineaire méthode différentielle
def cost_functiondif(coef): return np.sum((np.log(abs(vitesseprime)) - (coef[0]*np.log(C)+coef[1]))**2)
resdif = minimize(cost_functiondif,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(resdif.message)
opt_coefdif = resdif.x
residudif = np.log(abs(vitesseprime)) - (opt_coefdif[0]*np.log(C)+opt_coefdif[1])
r2dif = np.corrcoef([np.log(abs(vitesseprime)), opt_coefdif[0]*np.log(C)+opt_coefdif[1]])
Mdif=np.max(np.abs(residudif))
mudif=np.mean(residudif)
sigmadif=np.std(residudif)
txt_eqndif = f'ln(V) : ({opt_coefdif[0]:1.4g})*ln(C) + ({opt_coefdif[1]:1.4g}), (R^2 = {r2dif[0,1]**2:1.4f})'
print(txt_eqndif)

#Graphes et figures
#figure1
fig1 = plt.figure(num=1, figsize=[10,7])
plt.xlabel('temps (s)', fontsize = 12)
plt.ylabel(r'$C/C_{0}$', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$C/C_{0} = f(t)$', fontsize = 16)
plt.axis([0,t[len(t)-1],0,1.1*np.max(C)])
plt.plot(tprime, C, '+', label=r'C/C_0')
#plt.legend(loc='best')

#
##figure2
fig2 = plt.figure(num=2, figsize=[10,7])
ax1 = plt.subplot(211)
ax1.set_xlabel(r'temps (s)', fontsize = 12)
ax1.set_ylabel(r'$ln(C)$', fontsize = 12)
ax1.grid(linestyle='--')
ax1.set_title(r'$ln(C) = f(t)$', fontsize = 16)
ax1.plot(tprime[0:len(C)-1], np.log(C[0:len(C)-1]), '+--', label=r'ln( $C$')
ax1.plot(tprime,opt_coef[0]*tprime+opt_coef[1], 'r-' )
ax1.text(tprime[11],-4*opt_coef[1],txt_eqn, fontsize=12, color='red')
#
ax2 = plt.subplot(223)
ax2.plot(tprime,residu, '+' )
ax2.grid(linestyle='--')
ax2.set_xlabel('temps (s)', fontsize = 12)
ax2.set_ylabel('résidus', fontsize = 12)

#
ax3 = plt.subplot(224)
ax3.grid(linestyle='--')
ax3.hist(residu, facecolor='b', alpha=0.75)
ax3.text(-M,14,f'moyenne = {mu:1.4g}', fontsize=10)
ax3.text(-M,13,f'écart-type = {sigma:1.4f}', fontsize=10)
ax3.axis([-1.1*M,1.1*M,0,15])
#
#figure3
fig3 = plt.figure(num=3, figsize=[10,7])
plt.xlabel('c(mol/L)', fontsize = 12)
plt.ylabel(r'v (mol/L)', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$(v) = f(C)$', fontsize = 16)
plt.plot(C, vitesseprime, '+', label=r'vitesse')
plt.plot(C,opt_coefV[0]*C+opt_coefV[1], 'r-' )
plt.text(C[11],-4*opt_coefV[1],txt_eqnV, fontsize=12, color='red')
#plt.legend(loc='best')


##figure4
fig4 = plt.figure(num=4, figsize=[10,7])
ax1 = plt.subplot(211)
ax1.set_xlabel(r'ln(C)', fontsize = 12)
ax1.set_ylabel(r'$ln(v)$', fontsize = 12)
ax1.grid(linestyle='--')
ax1.set_title(r'$ln(v) = f(ln(C))$', fontsize = 16)
ax1.plot(np.log(C), np.log(vitesseprime), '+--', label=r'ln( $C$')
ax1.plot(np.log(C),opt_coefdif[0]*np.log(C)+opt_coefdif[1], 'r-' )
ax1.text(np.log(C[15]),2*opt_coefdif[1],txt_eqndif, fontsize=12, color='red')
#
ax2 = plt.subplot(223)
ax2.plot(tprime,residudif, '+' )
ax2.grid(linestyle='--')
ax2.set_xlabel('temps (s)', fontsize = 12)
ax2.set_ylabel('résidus', fontsize = 12)

#
ax3 = plt.subplot(224)
ax3.grid(linestyle='--')
ax3.hist(residudif, facecolor='b', alpha=0.75)
ax3.text(-Mdif,14,f'moyenne = {mu:1.4g}', fontsize=10)
ax3.text(-Mdif,13,f'écart-type = {sigma:1.4f}', fontsize=10)
ax3.axis([-1.1*Mdif,1.1*Mdif,0,15])

##Affichage des figures
if graphShow: plt.show()