import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import minimize

#déclaration des variables/paramètres
myFile = 'erythrosine2.txt'
graphShow = False # False/True
#

# initialisaton des variables  
temps = []
A = []

#lecture du fichier de données
data = open(myFile,'r')
for row in data:
    row = row.split('\t')
    temps.append(float(row[0]))
    A.append(float(row[1]))

t = np.array(temps)
A = np.array(A)

# calcul de la vitesse 
Vitesse = np.diff(A)/np.diff(t)

#calcul des paramètres de la régression lineaire méthode intégrale
def cost_function(coef): return np.sum((np.log(A) - (coef[0]*t+coef[1]))**2)
res = minimize(cost_function,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(res.message)
opt_coef = res.x
residu = np.log(A) - (opt_coef[0]*t+opt_coef[1])
r2 = np.corrcoef([np.log(A), opt_coef[0]*t+opt_coef[1]])
M=np.max(np.abs(residu))
mu=np.mean(residu)
sigma=np.std(residu)
txt_eqn = f'ln(A) : ({opt_coef[0]:1.4f})*t + ({opt_coef[1]:1.4f}), (R^2 = {r2[0,1]**2:1.4f})'
print(txt_eqn)

#calcul des paramètres de la régression lineaire v=kA
def cost_functionV(coef): return np.sum((Vitesse - (coef[0]*A[0:len(A)-1]+coef[1]))**2)
resV = minimize(cost_functionV,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(resV.message)
opt_coefV = resV.x
residuV = Vitesse - (opt_coefV[0]*A[0:len(A)-1]+opt_coefV[1])
r2V = np.corrcoef([Vitesse, opt_coefV[0]*A[0:len(A)-1]+opt_coefV[1]])
MV=np.max(np.abs(residuV))
muV=np.mean(residuV)
sigmaV=np.std(residuV)
txt_eqnV = f'V : ({opt_coefV[0]:1.4g})*A + ({opt_coefV[1]:1.4g}), (R^2 = {r2V[0,1]**2:1.4f})'
print(txt_eqnV)

#calcul des paramètres de la régression lineaire méthode différentielle
def cost_functiondif(coef): return np.sum((np.log(-Vitesse) - (coef[0]*np.log(A[0:len(A)-1])+coef[1]))**2)
resdif = minimize(cost_functiondif,[1,0], method='nelder-mead', options={'xatol': 1e-8, 'disp': False})
print(resdif.message)
opt_coefdif = resdif.x
residudif = np.log(-Vitesse) - (opt_coefdif[0]*np.log(A[0:len(A)-1])+opt_coefdif[1])
r2dif = np.corrcoef([np.log(-Vitesse), opt_coefdif[0]*np.log(A[0:len(A)-1])+opt_coefdif[1]])
Mdif=np.max(np.abs(residudif))
mudif=np.mean(residudif)
sigmadif=np.std(residudif)
txt_eqndif = f'ln(V) : ({opt_coefdif[0]:1.4g})*ln(A) + ({opt_coefdif[1]:1.4g}), (R^2 = {r2dif[0,1]**2:1.4f})'
print(txt_eqndif)


#Graphes et figures
#figure1
fig1 = plt.figure(num=1, figsize=[10,7])
plt.xlabel('temps (min)', fontsize = 12)
plt.ylabel(r'A', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$A = f(t)$', fontsize = 16)
plt.axis([0,t[len(t)-1],0,1.1*np.max(A)])
plt.plot(t, A, '+', label=r'A')
#plt.legend(loc='best')

#
##figure2
fig2 = plt.figure(num=2, figsize=[10,7])
ax1 = plt.subplot(211)
ax1.set_xlabel(r'temps (min)', fontsize = 12)
ax1.set_ylabel(r'$ln(A)$', fontsize = 12)
ax1.grid(linestyle='--')
ax1.set_title(r'$ln(A) = f(t)$', fontsize = 16)
ax1.plot(t, np.log(A), '+--', label=r'ln( $A$')
ax1.plot(t,opt_coef[0]*t+opt_coef[1], 'r-' )
ax1.text(t[7],1.5*opt_coef[1],txt_eqn, fontsize=12, color='red')
#
ax2 = plt.subplot(223)
ax2.plot(t,residu, '+' )
ax2.grid(linestyle='--')
ax2.set_xlabel('temps (min)', fontsize = 12)
ax2.set_ylabel('résidus', fontsize = 12)
ax2.axis([0,t[len(t)-1],-1.1*M,1.1*M])
#
ax3 = plt.subplot(224)
ax3.grid(linestyle='--')
ax3.hist(residu, facecolor='b', alpha=0.75)
ax3.text(-M,14,f'moyenne = {mu:1.4g}', fontsize=10)
ax3.text(-M,13,f'écart-type = {sigma:1.4f}', fontsize=10)
ax3.axis([-1.1*M,1.1*M,0,15])
#
##figure3
fig3 = plt.figure(num=3, figsize=[10,7])
plt.xlabel('temps (min)', fontsize = 12)
plt.ylabel(r'$1/A$ (L/mol)', fontsize = 12)
plt.grid(linestyle='--')
plt.title(r'$1/A = f(t)$', fontsize = 16)
plt.plot(t, 1/A, '+--', label=r'$1/A$')
#
##figure4
fig4 = plt.figure(num=4, figsize=[10,7])
ax1 = plt.subplot(211)
ax1.set_xlabel(r'A', fontsize = 12)
ax1.set_ylabel(r'$vitesse$', fontsize = 12)
ax1.grid(linestyle='--')
ax1.set_title(r'$v = f(A)$', fontsize = 16)
ax1.plot(A[0:len(A)-1], Vitesse, '+--', label=r'vitesse')
ax1.plot(A[0:len(A)-1],opt_coefV[0]*A[0:len(A)-1]+opt_coefV[1], 'r-' )
ax1.text(0.3*np.max(A),.5*(np.min(Vitesse)+np.max(Vitesse))/2,txt_eqnV, fontsize=12, color='red')
#
ax2 = plt.subplot(223)
ax2.plot(A[0:len(A)-1],residuV, '+' )
ax2.grid(linestyle='--')
ax2.set_xlabel('A', fontsize = 12)
ax2.set_ylabel('résidus vitesse', fontsize = 12)
ax2.axis([0,1.1*np.max(A),-1.1*MV,1.1*MV])
#
ax3 = plt.subplot(224)
ax3.grid(linestyle='--')
ax3.hist(residuV,int(np.rint(len(A)/4)), density=True, facecolor='b', alpha=0.75)
X=np.linspace(-1.1*MV,1.1*MV,100)
gauss = 1/(sigmaV*(2*np.pi)**0.5)*np.exp(-(X-muV)**2/(2*sigmaV**2))
ax3.plot(X,gauss)
ax3.text(-MV,1.3*np.max(gauss),f'moyenne = {muV:1.4g}', fontsize=10)
ax3.text(-MV,1.2*np.max(gauss),f'écart-type = {sigmaV:1.4g}', fontsize=10)
ax3.axis([-1.15*MV,1.15*MV,0,1.4*np.max(gauss)])
#
#figure5
fig5 = plt.figure(num=5, figsize=[10,7])
ax1 = plt.subplot(211)
ax1.set_xlabel(r'ln(A)', fontsize = 12)
ax1.set_ylabel(r'$ln(vitesse)$', fontsize = 12)
ax1.grid(linestyle='--')
ax1.set_title(r'$ln(v) = f(ln(A))$', fontsize = 16)
ax1.plot(np.log(A[0:len(A)-1]), np.log(abs(Vitesse)), '+--', label=r'vitesse')
ax1.plot(np.log(A[0:len(A)-1]),opt_coefdif[0]*np.log(A[0:len(A)-1])+opt_coefdif[1], 'r-' )
ax1.text (-3.2,-7.5, txt_eqndif, fontsize=12, color='red')
# ax1.text(0.3*np.max(np.log(A)),.5*(np.min(np.log(abs(Vitesse)))+np.max(np.log(abs(Vitesse))))/2,txt_eqndif, fontsize=12, color='red')
#
ax2 = plt.subplot(223)
ax2.plot(np.log(A[0:len(A)-1]),residudif, '+' )
ax2.grid(linestyle='--')
ax2.set_xlabel('ln(A)', fontsize = 12)
ax2.set_ylabel('résidus ln(vitesse)', fontsize = 12)
#ax2.axis([0,1.1*np.max(np.log(A)),-1.1*Mdif,1.1*Mdif])
#
ax3 = plt.subplot(224)
ax3.grid(linestyle='--')
ax3.hist(residudif,int(np.rint(len(np.log(A))/4)), density=True, facecolor='b', alpha=0.75)
Xdif=np.linspace(-1.1*Mdif,1.1*Mdif,100)
gaussdif = 1/(sigmadif*(2*np.pi)**0.5)*np.exp(-(Xdif-mudif)**2/(2*sigmadif**2))
ax3.plot(Xdif,gaussdif)
ax3.text(-Mdif,1.3*np.max(gaussdif),f'moyenne = {mudif:1.4g}', fontsize=10)
ax3.text(-Mdif,1.2*np.max(gaussdif),f'écart-type = {sigmadif:1.4g}', fontsize=10)
ax3.axis([-1.15*Mdif,1.15*Mdif,0,1.4*np.max(gaussdif)])

##Affichage des figures
if graphShow: plt.show()